@@ -40,14 +40,10 @@
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <glib-object.h>
-#define DBUS_API_SUBJECT_TO_CHANGE
-#include <dbus/dbus-glib.h>
-#include <dbus/dbus-glib-lowlevel.h>
 
 #include "gdm-common.h"
 
-#include "gdm-session.h"
-#include "gdm-session-direct.h"
+#include "gdm-session-enum-types.h"
 #include "gdm-welcome-session.h"
 
 #define DBUS_LAUNCH_COMMAND BINDIR "/dbus-launch"
@@ -64,10 +60,13 @@ struct GdmWelcomeSessionPrivate
         char           *command;
         GPid            pid;
 
+        GdmSessionVerificationMode verification_mode;
+
         char           *user_name;
         char           *group_name;
         char           *runtime_dir;
 
+        char           *session_id;
         char           *x11_display_name;
         char           *x11_display_seat_id;
         char           *x11_display_device;
@@ -79,13 +78,13 @@ struct GdmWelcomeSessionPrivate
         char           *dbus_bus_address;
         char           *server_dbus_path;
         char           *server_dbus_interface;
-        char           *server_env_var_name;
 
         char           *server_address;
 };
 
 enum {
         PROP_0,
+        PROP_VERIFICATION_MODE,
         PROP_X11_DISPLAY_NAME,
         PROP_X11_DISPLAY_SEAT_ID,
         PROP_X11_DISPLAY_DEVICE,
@@ -95,14 +94,13 @@ enum {
         PROP_USER_NAME,
         PROP_GROUP_NAME,
         PROP_RUNTIME_DIR,
-        PROP_SERVER_ADDRESS,
         PROP_COMMAND,
         PROP_SERVER_DBUS_PATH,
         PROP_SERVER_DBUS_INTERFACE,
-        PROP_SERVER_ENV_VAR_NAME
 };
 
 enum {
+        OPENED,
         STARTED,
         STOPPED,
         EXITED,
@@ -278,11 +276,6 @@ build_welcome_environment (GdmWelcomeSession *welcome_session,
                                      g_strdup ("DBUS_SESSION_BUS_ADDRESS"),
                                      g_strdup (welcome_session->priv->dbus_bus_address));
         }
-        if (welcome_session->priv->server_address != NULL) {
-                g_assert (welcome_session->priv->server_env_var_name != NULL);
-                g_hash_table_insert (hash, g_strdup (welcome_session->priv->server_env_var_name),
-                                     g_strdup (welcome_session->priv->server_address));
-        }
 
         g_hash_table_insert (hash, g_strdup ("XAUTHORITY"), g_strdup (welcome_session->priv->x11_authority_file));
         g_hash_table_insert (hash, g_strdup ("DISPLAY"), g_strdup (welcome_session->priv->x11_display_name));
@@ -694,49 +687,23 @@ on_session_setup_complete (GdmSession        *session,
 
         g_hash_table_iter_init (&iter, hash);
         while (g_hash_table_iter_next (&iter, &key, &value)) {
-                gdm_session_set_environment_variable (GDM_SESSION (welcome_session->priv->session), key, value);
+                gdm_session_set_environment_variable (welcome_session->priv->session, key, value);
         }
         g_hash_table_destroy (hash);
 
-        gdm_session_select_session_type (GDM_SESSION (welcome_session->priv->session), "LoginWindow");
-
-        gdm_session_authenticate (GDM_SESSION (welcome_session->priv->session), service_name);
-}
-
-static void
-on_session_authenticated (GdmSession        *session,
-                          const char        *service_name,
-                          GdmWelcomeSession *welcome_session)
-{
-        gdm_session_authorize (GDM_SESSION (welcome_session->priv->session),
-                               service_name);
-}
-
-static void
-on_session_authorized (GdmSession        *session,
-                       const char        *service_name,
-                       GdmWelcomeSession *welcome_session)
-{
-        gdm_session_accredit (GDM_SESSION (welcome_session->priv->session),
-                              service_name,
-                              GDM_SESSION_CRED_ESTABLISH);
-}
-
-static void
-on_session_accredited (GdmSession        *session,
-                       const char        *service_name,
-                       GdmWelcomeSession *welcome_session)
-{
-        gdm_session_open_session (GDM_SESSION (welcome_session->priv->session), service_name);
+        gdm_session_select_session_type (welcome_session->priv->session, "LoginWindow");
 }
 
 static void
 on_session_opened (GdmSession        *session,
                    const char        *service_name,
+                   const char        *session_id,
                    GdmWelcomeSession *welcome_session)
 {
-        gdm_session_start_session (GDM_SESSION (welcome_session->priv->session),
-                                   service_name);
+        welcome_session->priv->session_id = g_strdup (session_id);
+
+        g_signal_emit (G_OBJECT (welcome_session), signals [OPENED], 0);
+        gdm_session_start_session (welcome_session->priv->session, service_name);
 }
 
 static void
@@ -754,8 +721,7 @@ on_session_exited (GdmSession        *session,
                    int                exit_code,
                    GdmWelcomeSession *welcome_session)
 {
-        gdm_session_stop_conversation (GDM_SESSION (welcome_session->priv->session),
-                                       "gdm-welcome");
+        gdm_session_stop_conversation (welcome_session->priv->session, "gdm-welcome");
 
         g_signal_emit (G_OBJECT (welcome_session), signals [EXITED], 0, exit_code);
 }
@@ -765,8 +731,7 @@ on_session_died (GdmSession        *session,
                  int                signal_number,
                  GdmWelcomeSession *welcome_session)
 {
-        gdm_session_stop_conversation (GDM_SESSION (welcome_session->priv->session),
-                                       "gdm-welcome");
+        gdm_session_stop_conversation (welcome_session->priv->session, "gdm-welcome");
 
         g_signal_emit (G_OBJECT (welcome_session), signals [DIED], 0, signal_number);
 }
@@ -783,9 +748,7 @@ on_conversation_started (GdmSession        *session,
         log_path = g_build_filename (LOGDIR, log_file, NULL);
         g_free (log_file);
 
-        gdm_session_setup_for_program (GDM_SESSION (welcome_session->priv->session),
-                                       "gdm-welcome",
-                                       log_path);
+        gdm_session_setup_for_program (welcome_session->priv->session, "gdm-welcome", log_path);
         g_free (log_path);
 }
 
@@ -807,7 +770,7 @@ on_conversation_stopped (GdmSession        *session,
         }
 
         if (conversation_session != NULL) {
-                gdm_session_close (GDM_SESSION (conversation_session));
+                gdm_session_close (conversation_session);
                 g_object_unref (conversation_session);
         }
 }
@@ -822,6 +785,8 @@ gboolean
 gdm_welcome_session_start (GdmWelcomeSession *welcome_session)
 {
         gboolean          res;
+        struct passwd *passwd_entry;
+        uid_t uid;
 
         g_debug ("GdmWelcomeSession: Starting welcome...");
         res = start_dbus_daemon (welcome_session);
@@ -830,19 +795,28 @@ gdm_welcome_session_start (GdmWelcomeSession *welcome_session)
                 return FALSE;
         }
 
-        welcome_session->priv->session = GDM_SESSION (gdm_session_direct_new (NULL,
-                                                                              welcome_session->priv->x11_display_name,
-                                                                              welcome_session->priv->x11_display_hostname,
-                                                                              welcome_session->priv->x11_display_device,
-                                                                              welcome_session->priv->x11_display_seat_id,
-                                                                              welcome_session->priv->x11_authority_file,
-                                                                              welcome_session->priv->x11_display_is_local));
+        res = gdm_get_pwent_for_name (welcome_session->priv->user_name,
+                                      &passwd_entry);
 
-        g_signal_connect (GDM_SESSION (welcome_session->priv->session),
+        if (!res) {
+                return FALSE;
+        }
+
+        uid = passwd_entry->pw_uid;
+        welcome_session->priv->session = gdm_session_new (welcome_session->priv->verification_mode,
+                                                          uid,
+                                                          welcome_session->priv->x11_display_name,
+                                                          welcome_session->priv->x11_display_hostname,
+                                                          welcome_session->priv->x11_display_device,
+                                                          welcome_session->priv->x11_display_seat_id,
+                                                          welcome_session->priv->x11_authority_file,
+                                                          welcome_session->priv->x11_display_is_local);
+
+        g_signal_connect (welcome_session->priv->session,
                           "conversation-started",
                           G_CALLBACK (on_conversation_started),
                           welcome_session);
-        g_signal_connect (GDM_SESSION (welcome_session->priv->session),
+        g_signal_connect (welcome_session->priv->session,
                           "conversation-stopped",
                           G_CALLBACK (on_conversation_stopped),
                           welcome_session);
@@ -850,39 +824,25 @@ gdm_welcome_session_start (GdmWelcomeSession *welcome_session)
                           "setup-complete",
                           G_CALLBACK (on_session_setup_complete),
                           welcome_session);
-        g_signal_connect (welcome_session->priv->session,
-                          "authenticated",
-                          G_CALLBACK (on_session_authenticated),
-                          welcome_session);
-        g_signal_connect (welcome_session->priv->session,
-                          "authorized",
-                          G_CALLBACK (on_session_authorized),
-                          welcome_session);
-        g_signal_connect (welcome_session->priv->session,
-                          "accredited",
-                          G_CALLBACK (on_session_accredited),
-                          welcome_session);
         g_signal_connect (welcome_session->priv->session,
                           "session-opened",
                           G_CALLBACK (on_session_opened),
                           welcome_session);
-        g_signal_connect (GDM_SESSION (welcome_session->priv->session),
+        g_signal_connect (welcome_session->priv->session,
                           "session-started",
                           G_CALLBACK (on_session_started),
                           welcome_session);
-        g_signal_connect (GDM_SESSION (welcome_session->priv->session),
+        g_signal_connect (welcome_session->priv->session,
                           "session-exited",
                           G_CALLBACK (on_session_exited),
                           welcome_session);
-        g_signal_connect (GDM_SESSION (welcome_session->priv->session),
+        g_signal_connect (welcome_session->priv->session,
                           "session-died",
                           G_CALLBACK (on_session_died),
                           welcome_session);
 
-        gdm_session_start_conversation (GDM_SESSION (welcome_session->priv->session),
-                                        "gdm-welcome");
-        gdm_session_select_program (GDM_SESSION (welcome_session->priv->session),
-                                    welcome_session->priv->command);
+        gdm_session_start_conversation (welcome_session->priv->session, "gdm-welcome");
+        gdm_session_select_program (welcome_session->priv->session, welcome_session->priv->command);
         return TRUE;
 }
 
@@ -893,12 +853,10 @@ gdm_welcome_session_stop (GdmWelcomeSession *welcome_session)
                 gdm_signal_pid (welcome_session->priv->pid, SIGTERM);
         } else {
                 if (welcome_session->priv->session != NULL) {
-                        gdm_session_stop_conversation (GDM_SESSION (welcome_session->priv->session),
-                                                       "gdm-welcome");
-                        gdm_session_close (GDM_SESSION (welcome_session->priv->session));
+                        gdm_session_stop_conversation (welcome_session->priv->session, "gdm-welcome");
+                        gdm_session_close (welcome_session->priv->session);
 
-                        g_object_unref (welcome_session->priv->session);
-                        welcome_session->priv->session = NULL;
+                        g_clear_object (&welcome_session->priv->session);
                 } else {
                         stop_dbus_daemon (welcome_session);
                 }
@@ -909,14 +867,23 @@ gdm_welcome_session_stop (GdmWelcomeSession *welcome_session)
         return TRUE;
 }
 
-void
-gdm_welcome_session_set_server_address (GdmWelcomeSession *welcome_session,
-                                        const char        *address)
+GdmSession *
+gdm_welcome_session_get_session (GdmWelcomeSession *welcome_session)
 {
-        g_return_if_fail (GDM_IS_WELCOME_SESSION (welcome_session));
+        return welcome_session->priv->session;
+}
 
-        g_free (welcome_session->priv->server_address);
-        welcome_session->priv->server_address = g_strdup (address);
+char *
+gdm_welcome_session_get_session_id (GdmWelcomeSession *welcome_session)
+{
+        return g_strdup (welcome_session->priv->session_id);
+}
+
+static void
+_gdm_welcome_session_set_verification_mode (GdmWelcomeSession           *welcome_session,
+                                            GdmSessionVerificationMode   verification_mode)
+{
+        welcome_session->priv->verification_mode = verification_mode;
 }
 
 static void
@@ -1014,14 +981,6 @@ _gdm_welcome_session_set_command (GdmWelcomeSession *welcome_session,
         welcome_session->priv->command = g_strdup (name);
 }
 
-static void
-_gdm_welcome_session_set_server_env_var_name (GdmWelcomeSession *welcome_session,
-                                              const char        *name)
-{
-        g_free (welcome_session->priv->server_env_var_name);
-        welcome_session->priv->server_env_var_name = g_strdup (name);
-}
-
 static void
 gdm_welcome_session_set_property (GObject      *object,
                                   guint         prop_id,
@@ -1033,6 +992,9 @@ gdm_welcome_session_set_property (GObject      *object,
         self = GDM_WELCOME_SESSION (object);
 
         switch (prop_id) {
+        case PROP_VERIFICATION_MODE:
+                _gdm_welcome_session_set_verification_mode (self, g_value_get_enum (value));
+                break;
         case PROP_X11_DISPLAY_NAME:
                 _gdm_welcome_session_set_x11_display_name (self, g_value_get_string (value));
                 break;
@@ -1060,18 +1022,12 @@ gdm_welcome_session_set_property (GObject      *object,
         case PROP_RUNTIME_DIR:
                 _gdm_welcome_session_set_runtime_dir (self, g_value_get_string (value));
                 break;
-        case PROP_SERVER_ADDRESS:
-                gdm_welcome_session_set_server_address (self, g_value_get_string (value));
-                break;
         case PROP_SERVER_DBUS_PATH:
                 _gdm_welcome_session_set_server_dbus_path (self, g_value_get_string (value));
                 break;
         case PROP_SERVER_DBUS_INTERFACE:
                 _gdm_welcome_session_set_server_dbus_interface (self, g_value_get_string (value));
                 break;
-        case PROP_SERVER_ENV_VAR_NAME:
-                _gdm_welcome_session_set_server_env_var_name (self, g_value_get_string (value));
-                break;
         case PROP_COMMAND:
                 _gdm_welcome_session_set_command (self, g_value_get_string (value));
                 break;
@@ -1092,6 +1048,9 @@ gdm_welcome_session_get_property (GObject    *object,
         self = GDM_WELCOME_SESSION (object);
 
         switch (prop_id) {
+        case PROP_VERIFICATION_MODE:
+                g_value_set_enum (value, self->priv->verification_mode);
+                break;
         case PROP_X11_DISPLAY_NAME:
                 g_value_set_string (value, self->priv->x11_display_name);
                 break;
@@ -1119,18 +1078,12 @@ gdm_welcome_session_get_property (GObject    *object,
         case PROP_RUNTIME_DIR:
                 g_value_set_string (value, self->priv->runtime_dir);
                 break;
-        case PROP_SERVER_ADDRESS:
-                g_value_set_string (value, self->priv->server_address);
-                break;
         case PROP_SERVER_DBUS_PATH:
                 g_value_set_string (value, self->priv->server_dbus_path);
                 break;
         case PROP_SERVER_DBUS_INTERFACE:
                 g_value_set_string (value, self->priv->server_dbus_interface);
                 break;
-        case PROP_SERVER_ENV_VAR_NAME:
-                g_value_set_string (value, self->priv->server_env_var_name);
-                break;
         case PROP_COMMAND:
                 g_value_set_string (value, self->priv->command);
                 break;
@@ -1151,6 +1104,14 @@ gdm_welcome_session_class_init (GdmWelcomeSessionClass *klass)
 
         g_type_class_add_private (klass, sizeof (GdmWelcomeSessionPrivate));
 
+        g_object_class_install_property (object_class,
+                                         PROP_VERIFICATION_MODE,
+                                         g_param_spec_enum ("verification-mode",
+                                                            "verification mode",
+                                                            "verification mode",
+                                                            GDM_TYPE_SESSION_VERIFICATION_MODE,
+                                                            GDM_SESSION_VERIFICATION_MODE_LOGIN,
+                                                            G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
         g_object_class_install_property (object_class,
                                          PROP_X11_DISPLAY_NAME,
                                          g_param_spec_string ("x11-display-name",
@@ -1214,13 +1175,6 @@ gdm_welcome_session_class_init (GdmWelcomeSessionClass *klass)
                                                               "runtime dir",
                                                               NULL,
                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-        g_object_class_install_property (object_class,
-                                         PROP_SERVER_ADDRESS,
-                                         g_param_spec_string ("server-address",
-                                                              "server address",
-                                                              "server address",
-                                                              NULL,
-                                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
         g_object_class_install_property (object_class,
                                          PROP_SERVER_DBUS_PATH,
                                          g_param_spec_string ("server-dbus-path",
@@ -1235,13 +1189,6 @@ gdm_welcome_session_class_init (GdmWelcomeSessionClass *klass)
                                                               "server dbus interface",
                                                               NULL,
                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-        g_object_class_install_property (object_class,
-                                         PROP_SERVER_ENV_VAR_NAME,
-                                         g_param_spec_string ("server-env-var-name",
-                                                              "server env var name",
-                                                              "server env var name",
-                                                              NULL,
-                                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
         g_object_class_install_property (object_class,
                                          PROP_COMMAND,
                                          g_param_spec_string ("command",
@@ -1249,6 +1196,16 @@ gdm_welcome_session_class_init (GdmWelcomeSessionClass *klass)
                                                               "command",
                                                               NULL,
                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+        signals [OPENED] =
+                g_signal_new ("opened",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GdmWelcomeSessionClass, opened),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE,
+                              0);
         signals [STARTED] =
                 g_signal_new ("started",
                               G_OBJECT_CLASS_TYPE (object_class),
@@ -1333,8 +1290,8 @@ gdm_welcome_session_finalize (GObject *object)
         g_free (welcome_session->priv->server_address);
         g_free (welcome_session->priv->server_dbus_path);
         g_free (welcome_session->priv->server_dbus_interface);
-        g_free (welcome_session->priv->server_env_var_name);
         g_free (welcome_session->priv->dbus_bus_address);
+        g_free (welcome_session->priv->session_id);
 
         G_OBJECT_CLASS (gdm_welcome_session_parent_class)->finalize (object);
 }
